//////////////////////////////////////////////////////////////////////
// aft_serial_cfg.cpp - Device Manager Property Page for
//			A142/A144 Serial adapters.
//
// Author	:	David Rokhvarg
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "sdlacfg.h"

////////////////////////////////////////////////////////////////////////////////
//AFT cfg functions prototypes

int 
aft_serial_on_notify(	
	HWND		ParentHwnd,
	LPNMHDR		NmHdr,
	SETUPSTUFF	*Params
	);

void 
aft_serial_init(
	HKEY	hkey, 
	HWND	hdlg,
	GLOBALS	*cfg
	);

static int selected_ind=0;

static BOOL aft_serial_check_and_save_cfg(HWND hdlg);


///////////////////////////////////////////////////////////////////////////////

#define NUM_OF_OPER_MODE_OPTIONS	2

char* aft_serial_operational_mode_options[NUM_OF_OPER_MODE_OPTIONS] = 
	{	SDLA_DECODE_USEDBY_FIELD(API),
		SDLA_DECODE_USEDBY_FIELD(STACK)	//select this, if another driver
										//(like sprotocol.sys or wanpipe.sys) will run above. 
	};

static char *clock_source_options[] = {"External", "Internal", NULL};

#define NUM_OF_LINE_MODE_OPTIONS	2
static char* aft_line_mode_options[NUM_OF_LINE_MODE_OPTIONS] = {MODE_OPTION_HDLC, MODE_OPTION_BITSTRM};

static look_up_t connection_type_options[] = {
	{"Permanent (default)",	WANOPT_PERMANENT	},
	{"Switched",	WANOPT_SWITCHED		},
	{"Ondemand",	WANOPT_ONDEMAND		},
	{NULL,			0					}
};

static look_up_t data_encoding_options[] = {
	{"NRZ (default)",		WANOPT_NRZ	},
	{"NRZI",	WANOPT_NRZI	},
	{"FM0",		WANOPT_FM0	},
	{"FM1",		WANOPT_FM1	},
	{NULL,		0			}
};

static look_up_t line_idle_options[] = {
	{"Flag (default)",	WANOPT_IDLE_FLAG	},
	{"Mark",	WANOPT_IDLE_MARK	},
	{NULL,		0					}
};

//these are recommendations only:
#define MIN_RECOMMENDED_BITSTREAM_MTU	256
#define MIN_RECOMMENDED_HDLC_MTU		2048

///////////////////////////////////////////////////////////////////////////////
DLGPROC_RETURN_TYPE WINAPI AFT_Serial_PageDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
{							// PageDlgProc
	switch (msg)
	{						// process message
	case WM_NOTIFY:
		Debug(DBG_AFT_SERIAL_PAGE, "WM_NOTIFY");

		//If there is a problem with the new configuration 
		//refuse to close the property page and return TRUE.
		//If all checks are OK allow the closing.
        if(aft_serial_on_notify(hdlg, (NMHDR *)lParam, stuff) == HANDLE_ERROR)
		{
			Debug(DBG_AFT_SERIAL_PAGE, "Invalid Configuration, refusing to close!");
			SetWindowLong(hdlg, DWL_MSGRESULT, PSNRET_INVALID);
			return TRUE;
		}
		break;

	case WM_INITDIALOG:
		{
			if (okay == TRUE)
			{
				//initialize visual controls
				aft_serial_init(hkey, hdlg, &Globals_New_Cfg);
			}

			if (!okay)
			{					// no SampleInfo
				ShowWindow(GetDlgItem(hdlg, IDC_INFOLABEL), SW_HIDE);
				ShowWindow(GetDlgItem(hdlg, IDB_MOREINFO), SW_HIDE);
			}					// no SampleInfo
		}
		break;
		
   	case WM_DESTROY:
		Debug(DBG_AFT_SERIAL_PAGE, "Samclass:WM_DESTROY");
		{
			LPSTR infofile = (LPSTR) GetWindowLongPtr(hdlg, DWL_USER);
			if (infofile){
				GlobalFree((HGLOBAL) infofile);
			}
		}
		break;

	case WM_SHOWWINDOW:
		//This message received each time the Dialog shown/hidden.
		//Update visual controls with latest information stored in "Globals_New_Cfg".
		Debug(DBG_AFT_SERIAL_PAGE, "DBG_AFT_SERIAL_PAGE:WM_SHOWWINDOW");
		set_dialog_wanpipe_name(hdlg);
		aft_serial_init(hkey, hdlg, &Globals_New_Cfg);
		break;

    case WM_COMMAND:
		switch (LOWORD(wParam))
		{					// process command notification
		case BUTTON_HELP:
			WinHelp(GetActiveWindow(), "WanpipePnp.hlp", HELP_CONTEXT,
				IDH_LINE_CFG);
			break;

		case BUTTON_START_MONITOR:
			if(is_legacy_api_mode(Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].operational_mode) == false){
				//Interface name e.g.: WANPIPE1_IF1 - if names start from 1
				start_monitor(selected_ind + 1);
			}else{
				start_monitor(selected_ind);
			}
			break;
		}					// process command notification
		return TRUE;
	}						// process message

	return FALSE;
}							// PageDlgProc

///////////////////////////////////////////////////////////////////////////////

UINT CALLBACK AFT_Serial_PageCallbackProc(HWND junk, UINT msg, LPPROPSHEETPAGE p)
{							// PageCallbackProc
	if (msg == PSPCB_RELEASE && p->lParam)
	{
		Debug(DBG_AFT_SERIAL_PAGE, "PSPCB_RELEASE : AFT_Serial_PageCallbackProc()");


		//compare the configuration structures, restart if different.
		if((released_by_OK_button == 1) &&
		   memcmp(&Globals_Old_Cfg, &Globals_New_Cfg, sizeof(Globals_New_Cfg)))
		{
			if(yes_no_question("Device Configuration was changed. Do you want the Device to be restarted now?")
					== IDYES){
				restart_device();
			}else{
				MessageBox(GetActiveWindow(), (LPCTSTR)"Device was NOT restarted.",
					"Note for the user", MB_OK | MB_ICONINFORMATION | MB_SYSTEMMODAL );
			}
		}
		delete (SETUPSTUFF*) p->lParam;
		return 0;//return code is ignored
	}
	return TRUE;				// matters only for PSPCB_CREATE
}							// PageCallbackProc

///////////////////////////////////////////////////////////////////////////////

int aft_serial_on_notify(	HWND    ParentHwnd,
					LPNMHDR NmHdr,
					SETUPSTUFF * Params	)
{
    SP_DEVINSTALL_PARAMS spDevInstall = {0};
    TCHAR                friendlyName[LINE_LEN] ={0};
	BOOL	configuration_check_passed = FALSE;
    
    switch (NmHdr->code) 
	{
    case PSN_APPLY:
        //
        // Sent when the user clicks on Apply OR OK !!
        //
		configuration_check_passed = aft_serial_check_and_save_cfg(ParentHwnd);

		Debug(DBG_AFT_SERIAL_PAGE, "aft_serial_on_notify(): PSN_APPLY");

		released_by_OK_button = 1;

		if(configuration_check_passed == TRUE){
			return HANDLE_OK;
		}else{
			return HANDLE_ERROR;
		}
		break;

	case PSN_QUERYCANCEL:
		Debug(DBG_AFT_SERIAL_PAGE, "aft_serial_on_notify() : PSN_QUERYCANCEL");
		released_by_OK_button = 0;
    default:
		Debug(DBG_AFT_SERIAL_PAGE, "aft_serial_on_notify() : default case:");
        return HANDLE_DEFAULT;
    }
    return HANDLE_DEFAULT;   
}

///////////////////////////////////////////////////////////////////////////////

static BOOL aft_serial_check_and_save_cfg(HWND hdlg)
{	
	unsigned int	mtu, j;
	char			local_tmp_buff[TMP_BUFFER_LEN];

	Debug(DBG_AFT_SERIAL_PAGE, "aft_serial_check_and_save_cfg()");
	Debug(DBG_AFT_SERIAL_PAGE, "*********  Writing aft_serial Interface Configuration  ********\n");

	///////////////////////////////////////////////////////////////////
	//get MTU
	GetDlgItemText(hdlg, EDIT_MTU, Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].mtu, TMP_BUFFER_LEN);

	mtu = atoi(Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].mtu);
	if(mtu < MIN_AFT_MTU || mtu > MAX_AFT_MTU){

		_snprintf(local_tmp_buff, TMP_BUFFER_LEN, "Invalid value in \"MTU\". Minimum is %d, Maximum is %d.", MIN_AFT_MTU, MAX_AFT_MTU);

		MessageBox(GetActiveWindow(), local_tmp_buff, "Configuration Error", MB_OK|MB_ICONWARNING|MB_SYSTEMMODAL);
		return FALSE;
	}

	if(mtu % 4){
		_snprintf(local_tmp_buff, TMP_BUFFER_LEN, "Invalid value in \"MTU\". Must be divizible by 4.");

		MessageBox(GetActiveWindow(), local_tmp_buff, "Configuration Error", MB_OK|MB_ICONWARNING|MB_SYSTEMMODAL);
		return FALSE;
	}

	if(mtu < MIN_RECOMMENDED_HDLC_MTU){

		_snprintf(local_tmp_buff, TMP_BUFFER_LEN,
			"The recommended \"MTU\" for HDLC is %d. Current \"MTU\" is %d. Would you like to use the recommended \"MTU\"?",
			MIN_RECOMMENDED_HDLC_MTU, mtu);

		if(yes_no_question(local_tmp_buff) == IDYES){
			//set the MTU value
			_snprintf(	Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].mtu, TMP_BUFFER_LEN, "%d", MIN_RECOMMENDED_HDLC_MTU);
			SetDlgItemText(hdlg, EDIT_MTU, Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].mtu);
		}
	}

	snprintf(local_tmp_buff, TMP_BUFFER_LEN, "aft_logic_channel_%d_mtu", selected_ind);
	SetRegistryStringValue(hkey, local_tmp_buff, (char*)	Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].mtu);

	/////////////////////////////////////////////////////////////////////////////////////
	//get operational mode from the page
	j = SendDlgItemMessage(hdlg, COMBO_OPERATIONAL_MODE, CB_GETCURSEL, 0, 0);
	strcpy(	Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].operational_mode, aft_serial_operational_mode_options[j]);

	snprintf(local_tmp_buff, TMP_BUFFER_LEN, "aft_logic_channel_%d_operational_mode", selected_ind);
	SetRegistryStringValue(hkey, local_tmp_buff, (char*)Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].operational_mode);

	Debug(DBG_AFT_SERIAL_PAGE, "*********  Finished writing aft_serial Interface Configuration  ********\n");

	/////////////////////////////////////////////////////////////////////////////////////

	int	clock_source = SendDlgItemMessage(	hdlg, COMBO_SERIAL_CLOCK_SOURCE, CB_GETCURSEL, 0, 0);

	switch(clock_source)
	{
	case 0:
		clock_source = WANOPT_EXTERNAL;
		break;
	case 1:
		clock_source = WANOPT_INTERNAL;
		break;
	default:
		clock_source = WANOPT_EXTERNAL;
		break;
	}
	Globals_New_Cfg.wandev_conf.clocking = clock_source;

	/////////////////////////////////////////////////////////////////////////////////////
	//saving SERIAL Baud Rate
	if(clock_source == WANOPT_INTERNAL){
		GetDlgItemText(hdlg, EDIT_BAUD_RATE, local_tmp_buff, TMP_BUFFER_LEN);
		unsigned int BaudRate = atoi(local_tmp_buff);

		if(BaudRate > 2750000 /*PRI_MAX_BAUD_RATE_S514*/ || BaudRate < 8000){
			snprintf(local_tmp_buff, TMP_BUFFER_LEN, "Invalid 'Baud Rate'! Must be between %d and %d.", 
				2750000 /*PRI_MAX_BAUD_RATE_S514*/, 8000);

			MessageBox(GetActiveWindow(), local_tmp_buff, "Error", MB_OK|MB_ICONWARNING|MB_SYSTEMMODAL);
			return FALSE;
		}else{
			SetRegistryStringValue(hkey, "BaudRate", BaudRate);
			Globals_New_Cfg.wandev_conf.bps = BaudRate;
		}
	}
	/////////////////////////////////////////////////////////////////////////////////////

	//Save to registry only AFTER successful check!
	SetRegistryStringValue(hkey, "clock_source",	Globals_New_Cfg.wandev_conf.clocking);
	SetRegistryStringValue(hkey, "BaudRate",		Globals_New_Cfg.wandev_conf.bps);

	///////////////////////////////////////////////////////////////////////
	//get line mode
	j = SendDlgItemMessage(hdlg, COMBO_LINE_MODE, CB_GETCURSEL, 0, 0);
	strcpy(Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].line_mode, aft_line_mode_options[j]);

	//line mode
	snprintf(local_tmp_buff, TMP_BUFFER_LEN, "aft_logic_channel_%d_line_mode", selected_ind);
	SetRegistryStringValue(hkey, local_tmp_buff, (char*)Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].line_mode);

	///////////////////////////////////////////////////////////////////////
	//get CONNECTION_TYPE
	j = SendDlgItemMessage(hdlg, COMBO_SERIAL_CONNECTION_TYPE, CB_GETCURSEL, 0, 0);
	Globals_New_Cfg.serial_connection_type = connection_type_options[j].value;
	SetRegistryStringValue(hkey, "serial_connection_type",	Globals_New_Cfg.serial_connection_type);

	///////////////////////////////////////////////////////////////////////
	//get DATA_ENCODING
	j = SendDlgItemMessage(hdlg, COMBO_SERIAL_DATA_ENCODING, CB_GETCURSEL, 0, 0);
	Globals_New_Cfg.serial_line_coding = data_encoding_options[j].value;
	SetRegistryStringValue(hkey, "serial_line_coding",	Globals_New_Cfg.serial_line_coding);

	///////////////////////////////////////////////////////////////////////
	//get COMBO_SERIAL_LINE_IDLE
	j = SendDlgItemMessage(hdlg, COMBO_SERIAL_LINE_IDLE, CB_GETCURSEL, 0, 0);
	Globals_New_Cfg.serial_line_idle = line_idle_options[j].value;
	SetRegistryStringValue(hkey, "serial_line_idle",	Globals_New_Cfg.serial_line_idle);

	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////

void aft_serial_init(HKEY	hkey, HWND hdlg, GLOBALS* cfg)
{
	int i, n, current_selection_index;
	static char local_tmp_buff[TMP_BUFFER_LEN];

	//////////////////////////////////////////////////////////////////////////////////////////
	//set Interface name on the page
	if(is_legacy_api_mode(Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].operational_mode) == false){
		//Interface name e.g.: WANPIPE1_IF1 - if names start from 1
		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%s_IF%d", Globals_New_Cfg.DeviceName, selected_ind + 1);
	}else{
		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%s_IF%d", Globals_New_Cfg.DeviceName, selected_ind);
	}
	SetDlgItemText(hdlg, IF_NAME, local_tmp_buff);

	//////////////////////////////////////////////////////////////////////////////////////////
	//set the MTU value
	SetDlgItemText(hdlg, EDIT_MTU, Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].mtu);

	//////////////////////////////////////////////////////////////////////////////////////////
	//add strings to Operational Mode combobox
	SendDlgItemMessage(hdlg, COMBO_OPERATIONAL_MODE, CB_RESETCONTENT, 0, 0);
	for(i = 0; i < NUM_OF_OPER_MODE_OPTIONS; i++){
		SendDlgItemMessage(hdlg, COMBO_OPERATIONAL_MODE, CB_ADDSTRING, 0, (LPARAM)aft_serial_operational_mode_options[i]);
	}

	//make the current one selected
	i = SendDlgItemMessage(	hdlg, COMBO_OPERATIONAL_MODE, CB_FINDSTRING, -1, (LPARAM)Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].operational_mode);
		
	SendDlgItemMessage(hdlg, COMBO_OPERATIONAL_MODE, CB_SETCURSEL, (i != CB_ERR ? i : 0), 0);

	//////////////////////////////////////////////////////////////////////////////////////////
	//initialize Clock Source combo box
	SendDlgItemMessage(hdlg, COMBO_SERIAL_CLOCK_SOURCE, CB_RESETCONTENT, 0, 0);
	n=0;
	while(clock_source_options[n] != NULL)
	{	SendDlgItemMessage(	hdlg, COMBO_SERIAL_CLOCK_SOURCE, CB_ADDSTRING, 0, 
							(LPARAM)clock_source_options[n]);
		n++;
	}

	//make current one selected
	if(Globals_New_Cfg.wandev_conf.clocking == WANOPT_EXTERNAL)
	{	SendDlgItemMessage(hdlg, COMBO_SERIAL_CLOCK_SOURCE, CB_SETCURSEL, 0, 0);
	}else
	{	SendDlgItemMessage(hdlg, COMBO_SERIAL_CLOCK_SOURCE, CB_SETCURSEL, 1, 0);
	}

	//////////////////////////////////////////////////////////////////////////////////////////
	//Serial Baud Rate
	local_tmp_buff[0] = '\0';
	_itoa( Globals_New_Cfg.wandev_conf.bps, local_tmp_buff, 10 );
	SetDlgItemText(hdlg, EDIT_BAUD_RATE, local_tmp_buff);		
	//////////////////////////////////////////////////////////////////////////////////////////

	//////////////////////////////////////////////////////////////////////////////////////////
	//add strings to Line Mode combobox
	SendDlgItemMessage(hdlg, COMBO_LINE_MODE, CB_RESETCONTENT, 0, 0);
	for(i = 0; i < 2; i++){
		SendDlgItemMessage(hdlg, COMBO_LINE_MODE, CB_ADDSTRING, 0, (LPARAM)aft_line_mode_options[i]);
	}

	//make the current one selected
	i = SendDlgItemMessage(	hdlg, COMBO_LINE_MODE, CB_FINDSTRING, -1, (LPARAM)Globals_New_Cfg.aft_logic_ch_cfg[selected_ind].line_mode);

	SendDlgItemMessage(hdlg, COMBO_LINE_MODE, CB_SETCURSEL, (i != CB_ERR ? i : 0), 0);

	//////////////////////////////////////////////////////////////////////////////////////////
	//add strings to COMBO_SERIAL_CONNECTION_TYPE combobox
	SendDlgItemMessage(hdlg, COMBO_SERIAL_CONNECTION_TYPE, CB_RESETCONTENT, 0, 0);
	i = 0, current_selection_index = 0;
	while(connection_type_options[i].name != NULL){
		SendDlgItemMessage(hdlg, COMBO_SERIAL_CONNECTION_TYPE, CB_ADDSTRING, 0,	(LPARAM)connection_type_options[i].name);
		if(Globals_New_Cfg.serial_connection_type == connection_type_options[i].value){
			current_selection_index = i;
		}
		i++;
	}
	//make the current one selected
	SendDlgItemMessage(hdlg, COMBO_SERIAL_CONNECTION_TYPE, CB_SETCURSEL, current_selection_index, 0);

	//////////////////////////////////////////////////////////////////////////////////////////
	//add strings to COMBO_SERIAL_DATA_ENCODING combobox
	SendDlgItemMessage(hdlg, COMBO_SERIAL_DATA_ENCODING, CB_RESETCONTENT, 0, 0);
	i = 0, current_selection_index = 0;
	while(data_encoding_options[i].name != NULL){
		SendDlgItemMessage(hdlg, COMBO_SERIAL_DATA_ENCODING, CB_ADDSTRING, 0, (LPARAM)data_encoding_options[i].name);
		if(Globals_New_Cfg.serial_line_coding == data_encoding_options[i].value){
			current_selection_index = i;
		}
		i++;
	}
	//make the current one selected
	SendDlgItemMessage(hdlg, COMBO_SERIAL_DATA_ENCODING, CB_SETCURSEL, current_selection_index, 0);

	//////////////////////////////////////////////////////////////////////////////////////////
	//add strings to COMBO_SERIAL_LINE_IDLE combobox
	SendDlgItemMessage(hdlg, COMBO_SERIAL_LINE_IDLE, CB_RESETCONTENT, 0, 0);
	i = 0, current_selection_index = 0;
	while(line_idle_options[i].name != NULL){
		SendDlgItemMessage(hdlg, COMBO_SERIAL_LINE_IDLE, CB_ADDSTRING, 0, (LPARAM)line_idle_options[i].name);
		if(Globals_New_Cfg.serial_line_idle == line_idle_options[i].value){
			current_selection_index = i;
		}
		i++;
	}
	//make the current one selected
	SendDlgItemMessage(hdlg, COMBO_SERIAL_LINE_IDLE, CB_SETCURSEL, current_selection_index, 0);
}

//////////////////////////////////////////////////////////////////////////////

static void disable_enable_serial_controls(HWND hdlg, BOOL flag)
{
	EnableWindow( GetDlgItem(hdlg, EDIT_BAUD_RATE),						flag);
}
